import asyncio
from pathlib import Path
from shlex import quote
import venv

from . import const, utils, parser
from .utils.files import write_file
from .utils.process import run_cmd


async def exec():
    args = parser.parse_args()

    project_path = Path(args.dir).absolute()

    print('Checking project directory...')

    if not project_path.exists():
        print('Project directory not found, creating project directory')

        project_path.mkdir()
    else:
        print('Project directory found')

    name = args.name

    print('Creating Django project...')

    await run_cmd(('django-admin', 'startproject', name, str(project_path)))

    print('Django project created')

    print('Creating files...')

    venv_path = project_path / 'venv'

    tasks = [
        write_file(
            str(project_path / '.gitignore'),
            const.GITIGNORE
        ),
        write_file(
            str(project_path / '.env'),
            utils.generate_env(args.debug, args.hosts)
        ),
        write_file(
            str(project_path / 'env.example'),
            const.ENV_EXAMPLE
        ),
        utils.write_settings(str(project_path / args.name / 'settings.py')),
    ]

    if args.python:
        tasks.append(
            run_cmd([
                args.python,
                '-m',
                'venv',
                str(venv_path)
            ])
        )
    else:
        print(f'Creating virtual environment at {venv_path}')

        venv.create(str(venv_path), with_pip=True)

        print(f'Created virtual environment at {venv_path}')

    if args.git:
        tasks.append(run_cmd(['git', 'init', str(project_path)]))

    await asyncio.gather(*tasks)

    print('Created files')

    bin_path = venv_path / 'bin'

    pip_path = bin_path / 'pip'

    print('Installing required packages...')

    args.packages.extend(const.REQUIRED_PACKAGES)

    await utils.install_packages(
        str(pip_path),
        args.packages,
        *(('--no-compile',) if args.no_compile else tuple())
    )

    print('Installed required packages')

    requirements_path = project_path / 'requirements.txt'

    await utils.write_requirements(str(requirements_path), str(pip_path))

    tasks = []

    if args.git and args.commit:
        await run_cmd(
            (
                'git',
                'add',
                '-v',
                '.',
            ),
            str(project_path)
        )

        task = run_cmd(
            (
                'git',
                'commit',
                '-v',
                '-m',
                quote(args.commit_message)
            ),
            str(project_path)
        )

        tasks.append(task)

    if args.migrate:
        python_path = bin_path / 'python'

        manage_path = project_path / 'manage.py'

        task = run_cmd((
            str(python_path),
            quote(str(manage_path)),
            'migrate',
            '--no-input'
        ))

        tasks.append(task)

    await asyncio.gather(*tasks)


def main():
    asyncio.run(exec())


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        exit()



import os
import codecs
from setuptools import setup, find_packages

import build_django

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = build_django.__version__

AUTHOR = build_django.__author__

DESCRIPTION = 'Command line utility for building Django projects'

# Setting up
setup(
    name="build-django",
    version=VERSION,
    author=AUTHOR,
    description=DESCRIPTION,
    url='https://notabug.org/kapustlo/build-django',
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    keywords=['python', 'django', 'build', 'cli', 'generate', 'code'],
    entry_points={
        'console_scripts': [
            'build-django=build_django.__main__:main'
        ]
    },
    classifiers=[
        "Framework :: Django",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Topic :: Software Development :: Code Generators",
        "Environment :: Console",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.9",
        "Operating System :: POSIX :: Linux",
        "Natural Language :: English"
    ],
    python_requires=">=3.9",
    install_requires=(
        'django',
        'aiofiles'
    )
)





GITIGNORE: str = '/venv\n/static\n/media\n__pycache__\n*.sqlite*\n*.log\n.env'

ENV_EXAMPLE: str = '''
SECRET_KEY=

# List of comma separated values
ALLOWED_HOSTS=

# Remove if production
DEBUG=
'''

REQUIRED_PACKAGES: list[str] = (
    'wheel',
    'django',
    'django-environ',
    'gunicorn'
)
